#!/usr/bin/python
#
###########
#
# Description:heli_simulation.py Script to generate diorama animation
# by using light, sound and motion on
# helicopter simulation using GPIOZero
# Date: April 25,2020
# Revision:3.1
#
###########


# provides interface to input/output pins of the Pi
from gpiozero import LED, PWMLED, Motor
# pygame module function to start and stop audio
from pygame import mixer
import time					            # provides various time-related functions
import random                           # well randomness

# setup GPIO Lights
# define the helicopter light list
heli_light_list = (CABIN_LIGHT, STROBE_LIGHT, POSITION_LIGHT,
                   LANDING_LIGHT) = (LED(2), LED(3), LED(4), LED(27))

# setup the strobe
strobe_on = 0.05
strobe_off = 0.8

# setup GPIO Motor
FORWARD = 20  # set GPIO-20 as Input 1 of L293N IC
BACKWARD = 21  # set GPIO-21 as Input 2 of L293N IC

speeds = {
    'stopped': 0,
    'slow':    0.33,
    'medium':  0.40,
    'fast':    0.55
}

run_sequence = [
    ['stopped',  5],
    ['slow',    3],
    ['medium',  3],
    ['fast',    3],
    ['medium',  2],
    ['slow',    2],
    ['stopped',  0]
]

# setup Sound
mixer.init()
mixer.music.set_volume(0.5)  # value 0.0 to 1.0 set music volume
mixer.music.load("/home/pi/python/sounds/jet_engin.mp3")


while True:
    # helicopter lights on sequence, cabin, strobe, position, landing
    for light in heli_light_list:  # helicopter lights on sequence,cab_lit,stb_lit,pos_lit,lnd_lit
        if light == STROBE_LIGHT:
            # print(str(light))
            light.blink(on_time=strobe_on,
                        off_time=strobe_off, background=True)
            time.sleep(3)
        else:
            # print(light)
            light.on()
            time.sleep(3)

    # start engine sounds
    mixer.music.play()

    # engage rotor motion, the close() at the end is important
    rotor = Motor(FORWARD, BACKWARD, pwm=True)
    for speed, duration in run_sequence:
        print("Rotors " + speed +
              "(" + str(speeds[speed]) + ") for " + str(duration) + " seconds.")
        rotor.forward(speeds[speed])
        time.sleep(duration)
    rotor.stop()
    rotor.close()

    # stop engine sound
    mixer.music.stop()

    # Shutdown lights in the opposite order they were started
    for light in reversed(heli_light_list):
        light.off()
        time.sleep(3)
